;________________________________________________________________________
;
;                          P A C K E T I . A S M
;________________________________________________________________________
;
; Packet Receiving Code   by Byon Garrabrant  N6BG  byon@mail.com  Apr 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

	cblock				; variables
		packetin_last
		packetin_count
		packetin_length
		packetin_flags
		packetin_byte
		packetin_flag_count
		packetin_data
		packetin_size
		packetin_crc_hi
		packetin_crc_lo
		
	endc
	
#define	MODE1_OUT		PORTB, 1
#define	MODE0_OUT		PORTB, 2
#define	PACKET_IN_PORT		PORTB
#define	PACKET_IN_BIT		4
#define	BANK1			H'7F'
#define	PacketInNewByteFlag	packetin_flags, 0
#define	PacketInResetFlag	packetin_flags, 1
#define	PacketInDoneFlag	packetin_flags, 2
#define	PacketInActiveFlag	packetin_flags, 3
#define	PacketInValidFlag 	packetin_flags, 4
;________________________________________________________________________

PacketIn_Init
	bsf	STATUS, RP0		; Select Bank 1
	bcf	MODE0_OUT
	bcf	MODE1_OUT
	movlw	b'11011111'		; divide TMR0 by 256
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bcf	MODE1_OUT
	bcf	MODE0_OUT
	clrf	packetin_flags
	return
;________________________________________________________________________

PacketIn_Update
	btfss	INTCON, T0IF		; if not timer wrap
	 return 			;  done
	bcf	INTCON, T0IF		; clear timer wrap
	
	movlw	D'130'-2		; bump timer forward
	subwf	TMR0, F
	incfsz	packetin_count, F
	 incf	packetin_count, F	; if ( ++packetin_count > 255 )
	decf	packetin_count, F	;   packetin_count = 255
	
	movfw	PACKET_IN_PORT
	xorwf	packetin_last, W
	xorwf	packetin_last, F	; packetin_last = packet_in
	andlw	(1 << PACKET_IN_BIT)
	skpnz				; if ( packet_in didn't change )
	 return				;  done
	 
	movfw	packetin_count		
	movwf	packetin_length		; packetin_length = packetin_count
	clrf	packetin_count		; packetin_count = 0
	movlw	8			
	addwf	packetin_length, F	; round off packetin_length
	swapf	packetin_length, W
	
UpdatePacket				; apply the new bits
	xorlw	H'FF'
	andlw	H'0F'
	addwf	PCL, F
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Invalid
	goto	PacketIn_Flag
	goto	PacketIn_6
	call	PacketIn_Add1
	call	PacketIn_Add1
	call	PacketIn_Add1
	call	PacketIn_Add1
	goto	PacketIn_Add0
	goto	PacketIn_Invalid
	
PacketIn_Invalid
	btfsc	PacketInActiveFlag	; if we were active,
	 bsf	PacketInResetFlag	;  reset
	bcf	PacketInActiveFlag	; we are no longer active
	bcf	PacketInValidFlag
	clrf	packetin_flag_count
	return
	
PacketIn_6
	call	PacketIn_Add1
	call	PacketIn_Add1
	call	PacketIn_Add1
	call	PacketIn_Add1
	goto	PacketIn_Add1
	
PacketIn_Add0
	clrc
	goto	PacketIn_Add
PacketIn_Add1	
	setc
	clrf	packetin_flag_count	; we are not in flags
PacketIn_Add
	rrf	packetin_data, F	; save the bit
	rlf	packetin_data, W	; restore the bit
	movlw	1
	skpnc
	 xorwf	packetin_crc_lo, F	; crc ^= d & 1;
	clrc
	rrf	packetin_crc_hi, F	; crc >>= 1
	rrf	packetin_crc_lo, F
	skpc
	 goto	PacketIn_CRC_Done	; if shifted off 1
	movlw	H'08'			;  xor POLY 0x8408
	xorwf	packetin_crc_lo, F
	movlw	H'84'
	xorwf	packetin_crc_hi, F	
PacketIn_CRC_Done

	btfss	PacketInValidFlag
	 return
	decfsz	packetin_size, F
	 return
	movfw	packetin_data
	bsf	PacketInActiveFlag	; we are active
	bsf	PacketInNewByteFlag	; got a new incoming byte
	movwf	packetin_byte
	movlw	D'8'			; reset size count to 8
	movwf	packetin_size
	return
	
PacketIn_Flag
	movlw	D'8'
	movwf	packetin_size
	btfss	PacketInValidFlag	; if valid
	 goto	PacketIn_Not_Valid

	btfss	PacketInActiveFlag	; if active,
	 goto	PacketIn_Not_Active

	movlw	H'78'			; compare CRC to H'785C'
	xorwf	packetin_crc_hi, W
	skpz				; if bad
	 bsf	PacketInResetFlag	;  reset
	movlw	H'5C'
	xorwf	packetin_crc_lo, W
	skpz				; if bad
	 bsf	PacketInResetFlag	;  reset
	
	btfss	PacketInResetFlag	; if not reset
	 bsf	PacketInDoneFlag	;  packet is valid, we are done
	bcf	PacketInActiveFlag	; we are no longer active
	bcf	PacketInValidFlag	; we are not valid
	clrf	packetin_flag_count	; we are not in flags
	return

PacketIn_Not_Valid			; if not valid
	incf	packetin_flag_count, F	; count flags til valid
	movfw	packetin_flag_count
	xorlw	3			; if 3rd flag in a row
	skpnz
	 bsf	PacketInValidFlag	; we are valid
	 
PacketIn_Not_Active			; if not active
	movlw	H'FF'			; reset crc to 0xFFFF
	movwf	packetin_crc_hi
	movwf	packetin_crc_lo
	return
;________________________________________________________________________

